#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <lzma_adv.h>
#include <stdio.h>
#include <stdlib.h>


void
process(lzma_stream *strm, FILE *ifile, FILE *ofile,
		size_t size, lzma_action action)
{
	uint8_t in[size];
	uint8_t out[1U << 20];
	strm->avail_in = fread(in, 1, size, ifile);
	strm->next_in = in;
	strm->next_out = out;
	strm->avail_out = 1U << 20;
	if (lzma_code(strm, action) < 0) {
		fprintf(stderr, "run error\n");
		exit(2);
	}
	fwrite(out, 1, strm->next_out - out, ofile);
}


int
main()
{
	lzma_init();

	FILE *ifile = fopen("zzz", "r");
	FILE *ofile = fopen("zzz.lzma", "w");

	lzma_stream strm = LZMA_STREAM_INIT;

// 	lzma_extra e1 = { NULL, LZMA_EXTRA_FILENAME, 6, (uint8_t *)"7z.exe" };

	lzma_options_lzma lzma = {
		.dictionary_size = 1 << 17,
		.literal_context_bits = 3,
		.literal_pos_bits = 0,
		.pos_bits = 2,
		.mode = LZMA_MODE_BEST,
		.fast_bytes = 48,
		.match_finder = LZMA_MF_BT4,
		.match_finder_cycles = 0,
	};

	lzma_options_subblock subblock = {
		.allow_subfilters = 1,
		.flush = 0,
		.subblock_data_size = LZMA_SUBBLOCK_DATA_SIZE_DEFAULT,
		.subfilter = {
			.id = UINT64_MAX,
			.options = NULL,
		},
		.subfilter_uncompressed_size = UINT64_MAX,
	};

	lzma_options_stream stream = {
		.check = LZMA_CHECK_SHA256,
		.has_crc32 = 1,
		.alignment = 0,
	};
	stream.filters[0].id = LZMA_FILTER_SUBBLOCK;
	stream.filters[0].options = &subblock;
	stream.filters[1].id = LZMA_FILTER_LZMA;
	stream.filters[1].options = &lzma;
	stream.filters[2].id = UINT64_MAX;
	stream.metadata_filters[0].id = UINT64_MAX;
	stream.metadata_filters[0].options = &lzma;
	stream.metadata_filters[1].id = UINT64_MAX;
	stream.header = NULL; // &e1;
	stream.footer = NULL;
	const lzma_ret ret = lzma_stream_encoder_multi(&strm, &stream);
	if (ret != LZMA_OK) {
		fprintf(stderr, "error init: %d\n", ret);
		return 1;
	}

	process(&strm, ifile, ofile, 512, LZMA_RUN);

	subblock.flush = 1;
	subblock.subfilter.id = LZMA_FILTER_X86;

	process(&strm, ifile, ofile, 119296, LZMA_RUN);
	printf("%d\n", subblock.flush);

	subblock.flush = 1;
	subblock.subfilter.id = UINT64_MAX;

	process(&strm, ifile, ofile, 500000, LZMA_FINISH);
	printf("%d\n", subblock.flush);

	lzma_end(&strm);

	return 0;
}
